
\name{a00ad}
\alias{a00ad}

\title{a00ad: Library identification, details of implementation, major and minor marks}

\description{
a00ad prints information about the version of the NAG Library in use.

}

\usage{
a00ad()
}

\arguments{

}

\details{
R interface to the NAG Fortran routine A00ADF.}

\value{

\item{IMPL}{string

The implementation title which usually lists the target platform, operating 
system and compiler.


}
\item{PREC}{string

The working or basic precision of the implementation. Some functions may 
perform operations in reduced precision or additional precision, but the 
great majority will perform all operations in basic precision. See the 
introduction to the Fortran library for definitions of these precisions.


}
\item{PCODE}{string

The product code for the NAG Library implementation that is being used. 
The code has a discernible structure, but it is not necessary to know the 
details of this structure. The product code can be used to differentiate 
between individual product licence codes.


}
\item{MKMAJ}{integer

The major mark of the NAG Library implementation that is being used.


}
\item{MKMIN}{integer

The minor mark of the NAG Library implementation that is being used.


}
\item{HDWARE}{string

The target hardware for the NAG Library implementation that is being used.


}
\item{OPSYS}{string

The target operating system for the NAG Library implementation that is 
being used.


}
\item{FCOMP}{string

The compiler used to build the NAG Library implementation that is being 
used.


}
\item{VEND}{string

The subsidiary library, if any, that must be linked with the NAG Library 
implementation that is being used. If the implementation does not require 
a subsidiary library then the string \preformatted{ '(self-contained)' 
} will be returned in vend.


}
\item{LICVAL}{boolean

Specifies whether or not a valid licence has been found for the NAG Library 
implementation that is being used.


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/A00/a00adf.pdf}
}

\author{NAG}

\examples{

ans<-a00ad()
if (1){
writeLines(toString(cat(sprintf(" *** Start of NAG Toolbox for MATLAB implementation details ***\n\n","\n"))))


impl<-ans$IMPL

writeLines(sprintf(" Implementation title: \%s\n",impl,"\n"))


prec<-ans$PREC

writeLines(toString(cat(sprintf(" Precision: \%s\n",prec,"\n"))))


pcode<-ans$PCODE

writeLines(toString(cat(sprintf(" Product Code: \%s\n",pcode,"\n"))))


mkmaj<-ans$MKMAJ

mkmin<-ans$MKMIN

writeLines(toString(cat(sprintf(" Mark: \%d.\%d\n",mkmaj,mkmin,"\n"))))



vend<-ans$VEND
if (match(vend,"(self-contained)")==1){
writeLines(toString(cat(sprintf(" Vendor Library: None\n","\n"))))
}
else
{

writeLines(toString(cat(sprintf(" Vendor Library: \%s\n",vend,"\n"))))
}

writeLines(toString(cat(sprintf(" Applicable to:\n","\n"))))


hdware<-ans$HDWARE

writeLines(toString(cat(sprintf(" hardware - \%s\n",hdware,"\n"))))


opsys<-ans$OPSYS

writeLines(toString(cat(sprintf(" op. sys. - \%s\n",opsys,"\n"))))


fcomp<-ans$FCOMP

writeLines(toString(cat(sprintf(" compiler - \%s\n",fcomp,"\n"))))


writeLines(toString(cat(sprintf(" and compatible systems.\n\n","\n"))))


writeLines(toString(cat(sprintf(" *** End of NAG Toolbox for MATLAB implementation details ***\n\n","\n"))))

licval<-ans$LICVAL
if(licval){

pcode<-ans$PCODE

writeLines(toString(cat(sprintf(" A valid licence was found for \%s\n\n",pcode,"\n"))))


}else {

pcode<-ans$PCODE

writeLines(toString(cat(sprintf(" A valid licence was not found for \%s\n\n",pcode,"\n"))))

}
}
}

\keyword{math}
\concept{identification}
\concept{library}
