
\name{e04cb}
\alias{e04cb}

\title{e04cb: Unconstrained minimization using simplex algorithm, function of several 
variables using function values only}

\description{
e04cb minimizes a general function \eqn{{F}\left({x}\right)}{F(x)} of 
\eqn{{n}}{n} independent variables \eqn{{x} = \left({x}_{{1}}{x}_{{2}} 
\ldots {x}_{{n}}\right)^{T}}{x = (x_1x_2  . . .  x_n)^T} by the Nelder and Mead 
simplex method (see [Nelder J A Mead R (1965)]). Derivatives of the function 
need not be supplied.

}

\usage{
e04cb(x, tolf, tolx, funct, monit, maxcal,
      n = nrow(x))
}

\arguments{

\item{x}{double array

A guess at the position of the minimum. Note that the problem should be 
scaled so that the values of the \eqn{x[{i}]}{x[i]} are of order unity.


}
\item{tolf}{double

The error tolerable in the function values, in the following sense. If 
\eqn{{f}_{{i}}}{f_i} for \eqn{{i}={1} \ldots {n}+{1}}{i=1  . . .  n+1}, are 
the individual function values at the vertices of the current simplex, 
and if \eqn{{f}_{{m}}}{f_m} is the mean of these values, then you can request 
that e04cb should terminate if \deqn{\sqrt{\frac{{1}}{{{n} + {1}}}\sum_{{{i} 
= {1}}}^{{{n} + {1}}}\left({f}_{{i}} - {f}_{{m}}\right)^{{2}}} < tolf\mbox{.}}{sqrt((1)/(n + 1)\sum_i = 1^n + 1(f_i - f_m)^2) < tolf.}


}
\item{tolx}{double

The error tolerable in the spatial values, in the following sense. If \eqn{{L}{V}}{LV} 
denotes the \sQuote{linearized} volume of the current simplex, and if \eqn{{{L}{V}}_{{init}}}{LV_init} 
denotes the \sQuote{linearized} volume of the initial simplex, then you 
can request that e04cb should terminate if \deqn{\frac{{{L}{V}}}{{{{L}{V}}_{{init}}}} 
< tolx\mbox{.}}{(LV)/(LV_init) < tolx.}


}
\item{funct}{function

funct must evaluate the function \eqn{{F}}{F} at a specified point.  It should be tested separately before being used in conjunction with e04cb.

\code{(FC) = funct(n,xc)
}


}
\item{monit}{function

monit may be used to monitor the optimization process. It is invoked once every iteration.

\code{() = monit(fmin,fmax,sim,n,ncall,serror,vratio)
}


}
\item{maxcal}{integer

The maximum number of function evaluations to be allowed.


}
\item{n}{integer: \strong{default} = nrow(x)

\eqn{{n}}{n}, the number of variables.


}
}

\details{
R interface to the NAG Fortran routine E04CBF.}

\value{

\item{X}{double array

The value of \eqn{{x}}{x} corresponding to the function value in f.


}
\item{F}{double

The lowest function value found.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04cbf.pdf}
}

\author{NAG}

\examples{

ifail <- 0
funct = function(n, xc) {
    
    fc <- exp(xc[1]) \%*\% (4 \%*\% xc[1] \%*\% (xc[1] + xc[2]) + 2 \%*\% 
        xc[2] \%*\% (xc[2] + 1) + 1)
    list(FC = fc)
}
monit = function(fmin, fmax, sim, n, ncall, serror, 
    vratio) {
    
    
    if (user(1) != 0) {
        
        writeLines(toString(cat(sprintf("\nThere have been \%d function calls\n", 
            ncall, "\n"))))
        
        
        writeLines(toString(cat(sprintf("The smallest function value is \%10.4f\n", 
            fmin, "\n"))))
        
        
        writeLines(toString(cat(sprintf("The simplex is\n", "\n"))))
        
        
        writeLines(toString(cat(sprintf(sim, "\n"))))
        
        
        writeLines(toString(cat(sprintf("The standard deviation in function values at the vertices of the simplex is \%10.4f\n", 
            serror, "\n"))))
        
        
        writeLines(toString(cat(sprintf("The linearized volume ratio of the current simplex to the starting one is \%10.4f\n", 
            vratio, "\n"))))
        
        
    }
    list()
}

x <- matrix(c(-1, 1), nrow = 2, ncol = 1, byrow = TRUE)



tolf <- sqrt(x02aj()[["result"]])

tolx <- sqrt(tolf)

maxcal <- 100

user <- function(switch_integer) {
    switch(switch_integer, 0)
}

e04cb(x, tolf, tolx, funct, monit, maxcal) 

}

\keyword{math}

\keyword{optimize}
\concept{simplex}
\concept{unconstrained}
