
\name{e04dg}
\alias{e04dg}

\title{e04dg: Unconstrained minimum, preconditioned conjugate gradient algorithm, function 
of several variables using first derivatives (comprehensive)}

\description{
e04dg minimizes an unconstrained nonlinear function of several variables 
using a pre-conditioned, limited memory quasi-Newton conjugate gradient 
method. First derivatives (or an \sQuote{acceptable} finite difference 
approximation to them) are required. It is intended for use on large scale 
problems.


}

\usage{
e04dg(objfun, x, optlist,
      n = nrow(x))
}

\arguments{

\item{objfun}{function

objfun must calculate the objective function \eqn{{F}\left({x}\right)}{F(x)} and possibly its gradient as well for a specified \eqn{{n}}{n} element vector \eqn{{x}}{x}.

\code{(MODE,OBJF,OBJGRD) = objfun(mode,n,x,nstate)
}


}
\item{x}{double array

An initial estimate of the solution.


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Estimated Optimal Function Value} \tab \emph{double} \tab  \cr
\code{Function Precision} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.9}}}{ = \epsilon^0.9} \cr
\code{Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{5}{n}})}{ = max(50, 5n)} \cr
\code{Iters} \tab \emph{} \tab  \cr
\code{Itns} \tab \emph{} \tab  \cr
\code{Linesearch Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{List} \tab \emph{} \tab Default for \eqn{e04dg = list}{e04dg = list} \cr
\code{Nolist} \tab \emph{} \tab Default for \eqn{e04dg = nolist}{e04dg = nolist} \cr
\code{Maximum Step Length} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}_{{R}}^{{0.8}}}{ = \epsilon_R^0.8} \cr
\code{Print Level} \tab \emph{integer} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Start Objective Check at Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Objective Check at Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Verify Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Verify} \tab \emph{} \tab  \cr
\code{Verify Gradients} \tab \emph{} \tab  \cr
\code{Verify Objective Gradients} \tab \emph{} \tab  \cr
}
}
\item{n}{integer: \strong{default} = nrow(x)

\eqn{{n}}{n}, the number of variables.


}
}

\details{
R interface to the NAG Fortran routine E04DGF.}

\value{

\item{ITER}{integer

The total number of iterations performed.


}
\item{OBJF}{double

The value of the objective function at the final iterate.


}
\item{OBJGRD}{double array

The gradient of the objective function at the final iterate (or its finite 
difference approximation).


}
\item{X}{double array

The final estimate of the solution.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04dgf.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
objfun = function(mode, n, x, nstate) {
    
    objgrd <- as.matrix(mat.or.vec(2, 1))
    expx1 <- exp(x[1])
    objf <- expx1 \%*\% (4 \%*\% x[1]^2 + 2 \%*\% x[2]^2 + 4 \%*\% x[1] \%*\% 
        x[2] + 2 \%*\% x[2] + 1)
    
    if (mode == 2) {
        
        objgrd[1] <- 4 \%*\% expx1 \%*\% (2 \%*\% x[1] + x[2]) + objf
        
        objgrd[2] <- 2 \%*\% expx1 \%*\% (2 \%*\% x[2] + 2 \%*\% x[1] + 
            1)
        
    }
    else {
        
        objgrd <- as.matrix(mat.or.vec(2, 1))
    }
    list(MODE = as.integer(mode), OBJF = objf, OBJGRD = as.matrix(objgrd))
}

x <- matrix(c(-1, 1), nrow = 2, ncol = 1, byrow = TRUE)



e04dg(objfun, x, optlist) 

}

\keyword{math}

\keyword{optimize}
\concept{conjugate}
\concept{derivative}
\concept{gradient}
\concept{minimum}
\concept{preconditioned}
\concept{unconstrained}
