
\name{e04nc}
\alias{e04nc}

\title{e04nc: Convex QP problem or linearly-constrained linear least squares problem 
(dense)}

\description{
e04nc solves linearly constrained linear least squares problems and convex 
quadratic programming problems. It is not intended for large sparse problems.


}

\usage{
e04nc(c, bl, bu, cvec, istate, kx, x, a, b, optlist,
      m = nrow(a),
      n = nrow(kx),
      nclin = nrow(c))
}

\arguments{

\item{c}{double array

The \eqn{{i}}{i}th row of c must contain the coefficients of the \eqn{{i}}{i}th 
general constraint for \eqn{{i}={1} \ldots nclin}{i=1  . . .  nclin}.


}
\item{bl}{double array}
\item{bu}{double array

Bl must contain the lower bounds and bu the upper bounds, for all the constraints, 
in the following order. The first \eqn{{n}}{n} elements of each array must 
contain the bounds on the variables, and the next \eqn{{n}_{{L}}}{n_L} 
elements must contain the bounds for the general linear constraints (if 
any). To specify a nonexistent lower bound (i.e., \eqn{{l}_{{j}} = { - 
{\infty}}}{l_j =  -  infinity}), set \eqn{bl[{j}] \leq - {bigbnd}}{bl[j]  <=  - bigbnd}, 
and to specify a nonexistent upper bound (i.e., \eqn{{u}_{{j}} = { + {\infty}}}{u_j =  +  infinity}), 
set \eqn{bu[{j}] \geq {bigbnd}}{bu[j]  >=  bigbnd}; the default value of \eqn{{bigbnd}}{bigbnd} 
is \eqn{{10}^{{20}}}{10^20}, but this may be changed by the optional argument 
infiniteboundsize. To specify the \eqn{{j}}{j}th constraint as an equality, 
set \eqn{bu[{j}] = bl[{j}] = {\beta}}{bu[j] = bl[j] = \beta}, say, where \eqn{\mathrm{abs}({\beta}) 
< {bigbnd}}{abs(\beta) < bigbnd}.


}
\item{cvec}{double array

The coefficients of the explicit linear term of the objective function.


}
\item{istate}{integer array

Need not be set if the (default) optional argument coldstart is used.


}
\item{kx}{integer array

Need not be initialized for problems of type FP, LP, QP1, QP2, LS1 (the 
default) or LS2.


}
\item{x}{double array

An initial estimate of the solution.


}
\item{a}{double array

The array a must contain the matrix \eqn{{A}}{A} as specified in \emph{table1} 
(see the Description in Fortran library documentation).


}
\item{b}{double array

The \eqn{{m}}{m} elements of the vector of observations.


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Cold Start} \tab \emph{} \tab Default \cr
\code{Warm Start} \tab \emph{} \tab  \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.01}}{ = 0.01} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Feasibility Phase Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{5}\left({n} + {n}_{{L}}\right)})}{ = max(50, 5(n + n_L))} \cr
\code{Optimality Phase Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{5}\left({n} + {n}_{{L}}\right)})}{ = max(50, 5(n + n_L))} \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Hessian} \tab \emph{no} \tab Default \eqn{\mbox{} = {NO}}{ = NO}
 \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Infinite Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}({bigbnd}, {10}^{{20}})}{ = max(bigbnd, 10^20)} \cr
\code{Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{5}\left({n} + {n}_{{L}}\right)})}{ = max(50, 5(n + n_L))} \cr
\code{Iters} \tab \emph{} \tab  \cr
\code{Itns} \tab \emph{} \tab  \cr
\code{List} \tab \emph{} \tab Default for \eqn{e04nc = list}{e04nc = list} \cr
\code{Nolist} \tab \emph{} \tab Default for \eqn{e04nc = nolist}{e04nc = nolist} \cr
\code{Monitoring File} \tab \emph{integer} \tab Default \eqn{\mbox{} = { - {1}}}{ =  - 1} \cr
\code{Print Level} \tab \emph{integer} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Problem Type} \tab \emph{string} \tab Default \eqn{=}{=} LS1 \cr
\code{Rank Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {100}{\epsilon}}{ = 100\epsilon} or \eqn{{10}\sqrt{{\epsilon}}}{10sqrt(\epsilon)} (see below) \cr
}
}
\item{m}{integer: \strong{default} = nrow(a)

\eqn{{m}}{m}, the number of rows in the matrix \eqn{{A}}{A}. If the problem 
is specified as type FP or LP, m is not referenced and is assumed to be 
zero.


}
\item{n}{integer: \strong{default} = nrow(kx)

\eqn{{n}}{n}, the number of variables.


}
\item{nclin}{integer: \strong{default} = nrow(c)

\eqn{{n}_{{L}}}{n_L}, the number of general linear constraints.


}
}

\details{
R interface to the NAG Fortran routine E04NCF.}

\value{

\item{ISTATE}{integer array

The status of the constraints in the working set at the point returned 
in x. The significance of each possible value of \eqn{istate[{j}]}{istate[j]} 
is as follows:


}
\item{KX}{integer array

Defines the order of the columns of a with respect to the ordering of x, 
as described above.


}
\item{X}{double array

The point at which e04nc terminated. If \eqn{\mathrm{ifail} =0}{ifail =0}, 
\eqn{\mathrm{ifail} =1}{ifail =1}, \eqn{\mathrm{ifail} =4}{ifail =4}, x 
contains an estimate of the solution.


}
\item{A}{double array

If \eqn{hessian=\code{NO}}{hessian=NO} and the problem is of type LS 
or QP, a contains the upper triangular Cholesky factor \eqn{{R}}{R} of 
eqn8 (see the Fortran library documentation), with columns ordered as 
indicated by kx. If \eqn{hessian=\code{YES}}{hessian=YES} and the problem 
is of type LS or QP, a contains the upper triangular Cholesky factor \eqn{{R}}{R} 
of the Hessian matrix \eqn{{H}}{H}, with columns ordered as indicated by 
kx. In either case \eqn{{R}}{R} may be used to obtain the variance-covariance 
matrix or to recover the upper triangular factor of the original least 
squares matrix.


}
\item{B}{double array

The transformed residual vector of equation eqn10 (see the Fortran library 
documentation).


}
\item{ITER}{integer

The total number of iterations performed.


}
\item{OBJ}{double

The value of the objective function at \eqn{{x}}{x} if \eqn{{x}}{x} is 
feasible, or the sum of infeasibiliites at \eqn{{x}}{x} otherwise. If the 
problem is of type FP and \eqn{{x}}{x} is feasible, obj is set to zero.


}
\item{CLAMDA}{double array

The values of the Lagrange multipliers for each constraint with respect 
to the current working set. The first \eqn{{n}}{n} elements contain the 
multipliers for the bound constraints on the variables, and the next \eqn{{n}_{{L}}}{n_L} 
elements contain the multipliers for the general linear constraints (if 
any). If \eqn{istate[{j}] = {0}}{istate[j] = 0} (i.e., constraint \eqn{{j}}{j} 
is not in the working set), \eqn{clamda[{j}]}{clamda[j]} is zero. If \eqn{{x}}{x} 
is optimal, \eqn{clamda[{j}]}{clamda[j]} should be non-negative if \eqn{istate[{j}] 
= {1}}{istate[j] = 1}, non-positive if \eqn{istate[{j}] = {2}}{istate[j] = 2} 
and zero if \eqn{istate[{j}] = {4}}{istate[j] = 4}.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04ncf.pdf}
}

\author{NAG}

\examples{

optlist<-list()

ifail<-0

c<-matrix(c(1,1,1,1,1,1,1,1,4,1,2,3,4,-2,1,1,1,1,1,-1,1,-1,1,1,1,1,1),nrow=3,ncol=9,byrow=TRUE)



bl<-matrix(c(0,0,-9.999999999999999e+24,0,0,0,0,0,0,2,-9.999999999999999e+24,1),nrow=12,ncol=1,byrow=TRUE)



bu<-matrix(c(2,2,2,2,2,2,2,2,2,9.999999999999999e+24,2,4),nrow=12,ncol=1,byrow=TRUE)



cvec<-matrix(c(0),nrow=1,ncol=1,byrow=TRUE)



istate<-as.matrix(mat.or.vec(12,1))

kx<-as.matrix(mat.or.vec(9,1))

x<-matrix(c(1,0.5,0.3333,0.25,0.2,0.1667,0.1428,0.125,0.1111),nrow=9,ncol=1,byrow=TRUE)



a<-matrix(c(1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,0,0,1,1,3,1,1,1,-1,-1,-3,1,1,1,4,1,1,1,1,1,1,1,1,3,1,1,1,1,1,1,1,2,1,1,0,0,0,-1,1,1,1,1,0,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,0,1,1,1,2,2,3,1,0,1,1,1,1,0,2,2),nrow=10,ncol=9,byrow=TRUE)



b<-matrix(c(1,1,1,1,1,1,1,1,1,1),nrow=10,ncol=1,byrow=TRUE)



e04nc(c,bl,bu,cvec,istate,kx,x,a,b,optlist)

}

\keyword{math}

\keyword{optimize}
\concept{convex}
\concept{least squares}
\concept{linear}
\concept{linearly-constrained}
\concept{QP}
