
\name{e04ug}
\alias{e04ug}

\title{e04ug: NLP problem (sparse)}

\description{
e04ug solves sparse nonlinear programming problems.


}

\usage{
e04ug(confun, objfun, n, m, ncnln, nonln, njnln, iobj, a, ha, ka, bl, bu, start, names, ns, xs, istate, clamda, optlist,
      nnz = nrow(a),
      nname = nrow(names),
      leniz = (1000),
      lenz = (1000))
}

\arguments{

\item{confun}{function

confun must calculate the vector \eqn{{F}\left({x}\right)}{F(x)} of nonlinear constraint functions and (optionally) its Jacobian \eqn{\left( = \frac{{ \partial {F}}}{{ \partial {x}}}\right)}{( = ( \partial F)/( \partial x))} for a specified \eqn{{n}_{{1}}''}{n_1''} (\eqn{\mbox{} \leq {n}}{  <=  n}) element vector \eqn{{x}}{x}.  If there are no nonlinear constraints (i.e., \eqn{ncnln = {0}}{ncnln = 0}), confun will never be called by e04ug and confun may be the dummy function e04ugm.  (e04ugm is included in the NAG Library.) If there are nonlinear constraints, the first call to confun will occur before the first call to objfun.

\code{(MODE,F,FJAC) = confun(mode,ncnln,njnln,nnzjac,x,fjac,nstate)
}


}
\item{objfun}{function

objfun must calculate the nonlinear part of the objective function \eqn{{f}\left({x}\right)}{f(x)} and (optionally) its gradient \eqn{\left( = \frac{{ \partial {f}}}{{ \partial {x}}}\right)}{( = ( \partial f)/( \partial x))} for a specified \eqn{{n}_{{1}}'}{n_1'} (\eqn{\mbox{} \leq {n}}{  <=  n}) element vector \eqn{{x}}{x}.  If there are no nonlinear objective variables (i.e., \eqn{nonln = {0}}{nonln = 0}), objfun will never be called by e04ug and objfun may be the dummy function e04ugn.  (e04ugn is included in the NAG Library.)

\code{(MODE,OBJF,OBJGRD) = objfun(mode,nonln,x,objgrd,nstate)
}


}
\item{n}{integer

\eqn{{n}}{n}, the number of variables (excluding slacks). This is the number 
of columns in the full Jacobian matrix \eqn{{A}}{A}.


}
\item{m}{integer

\eqn{{m}}{m}, the number of general constraints (or slacks). This is the 
number of rows in \eqn{{A}}{A}, including the free row (if any; see iobj). 
Note that \eqn{{A}}{A} must contain at least one row. If your problem has 
no constraints, or only upper and lower bounds on the variables, then you 
must include a dummy \sQuote{free} row consisting of a single (zero) element 
subject to \sQuote{infinite} upper and lower bounds. Further details can 
be found under the descriptions for iobj, nnz, a, ha, ka, bl and bu.


}
\item{ncnln}{integer

\eqn{{n}_{{N}}}{n_N}, the number of nonlinear constraints.


}
\item{nonln}{integer

\eqn{{n}_{{1}}'}{n_1'}, the number of nonlinear objective variables. If 
the objective function is nonlinear, the leading \eqn{{n}_{{1}}'}{n_1'} 
columns of \eqn{{A}}{A} belong to the nonlinear objective variables. (See 
also the description for njnln.)


}
\item{njnln}{integer

\eqn{{n}_{{1}}''}{n_1''}, the number of nonlinear Jacobian variables. If 
there are any nonlinear constraints, the leading \eqn{{n}_{{1}}''}{n_1''} 
columns of \eqn{{A}}{A} belong to the nonlinear Jacobian variables. If 
\eqn{{n}_{{1}}' > {0}}{n_1' > 0} and \eqn{{n}_{{1}}'' > {0}}{n_1'' > 0}, 
the nonlinear objective and Jacobian variables overlap. The total number 
of nonlinear variables is given by \eqn{\bar{{n}} = \mathrm{max}({{n}_{{1}}'}, 
{{n}_{{1}}''})}{barn = max(n_1', n_1'')}.


}
\item{iobj}{integer

If \eqn{iobj > ncnln}{iobj > ncnln}, row iobj of \eqn{{A}}{A} is a free 
row containing the nonzero elements of the linear part of the objective 
function. 

\eqn{iobj = {0}}{iobj = 0}: There is no free row.

\eqn{iobj = { - {1}}}{iobj =  - 1}: There is a dummy \sQuote{free} row.




}
\item{a}{double array

The nonzero elements of the Jacobian matrix \eqn{{A}}{A}, ordered by increasing 
column index. Since the constraint Jacobian matrix \eqn{{J}\left({x}''\right)}{J(x'')} 
must always appear in the top left-hand corner of \eqn{{A}}{A}, those elements 
in a column associated with any nonlinear constraints must come before 
any elements belonging to the linear constraint matrix \eqn{{G}}{G} and 
the free row (if any; see iobj).


}
\item{ha}{integer array

\eqn{ha[{i}]}{ha[i]} must contain the row index of the nonzero element 
stored in \eqn{a[{i}]}{a[i]} for \eqn{{i}={1} \ldots nnz}{i=1  . . .  nnz}. 
The row indices for a column may be supplied in any order subject to the 
condition that those elements in a column associated with any nonlinear 
constraints must appear before those elements associated with any linear 
constraints (including the free row, if any). Note that confun must define 
the Jacobian elements in the same order. If \eqn{iobj = { - {1}}}{iobj =  - 1}, 
set \eqn{ha[{1}] = {1}}{ha[1] = 1}.


}
\item{ka}{integer array

\eqn{ka[{j}]}{ka[j]} must contain the index in a of the start of the \eqn{{j}}{j}th 
column for \eqn{{j}={1} \ldots n}{j=1  . . .  n}. To specify the \eqn{{j}}{j}th 
column as empty, set \eqn{ka[{j}] = ka[{j}+{1}]}{ka[j] = ka[j+1]}. Note 
that the first and last elements of ka must be such that \eqn{ka[{1}] = 
{1}}{ka[1] = 1} and \eqn{ka[n+{1}] = nnz + {1}}{ka[n+1] = nnz + 1}. If 
\eqn{iobj = { - {1}}}{iobj =  - 1}, set \eqn{ka[{j}] = {2}}{ka[j] = 2} 
for \eqn{{j}={2} \ldots n}{j=2  . . .  n}.


}
\item{bl}{double array

\eqn{{l}}{l}, the lower bounds for all the variables and general constraints, 
in the following order. The first n elements of bl must contain the bounds 
on the variables \eqn{{x}}{x}, the next ncnln elements the bounds for the 
nonlinear constraints \eqn{{F}\left({x}\right)}{F(x)} (if any) and the 
next (\eqn{m - ncnln}{m - ncnln}) elements the bounds for the linear constraints 
\eqn{{G}{x}}{Gx} and the free row (if any). To specify a nonexistent lower 
bound (i.e., \eqn{{l}_{{j}} = { - {\infty}}}{l_j =  -  infinity}), set \eqn{bl[{j}] 
\leq - {bigbnd}}{bl[j]  <=  - bigbnd}. To specify the \eqn{{j}}{j}th constraint 
as an \emph{equality}, set \eqn{bl[{j}] = bu[{j}] = {\beta}}{bl[j] = bu[j] = \beta}, 
say, where \eqn{\mathrm{abs}({\beta}) < {bigbnd}}{abs(\beta) < bigbnd}. If \eqn{iobj 
= { - {1}}}{iobj =  - 1}, set \eqn{bl[n+\mathrm{abs}\left(iobj\right)] 
\leq - {bigbnd}}{bl[n+abs(iobj)]  <=  - bigbnd}.


}
\item{bu}{double array

\eqn{{u}}{u}, the upper bounds for all the variables and general constraints, 
in the following order. The first n elements of bu must contain the bounds 
on the variables \eqn{{x}}{x}, the next ncnln elements the bounds for the 
nonlinear constraints \eqn{{F}\left({x}\right)}{F(x)} (if any) and the 
next (\eqn{m - ncnln}{m - ncnln}) elements the bounds for the linear constraints 
\eqn{{G}{x}}{Gx} and the free row (if any). To specify a nonexistent upper 
bound (i.e., \eqn{{u}_{{j}} = { + {\infty}}}{u_j =  +  infinity}), set \eqn{bu[{j}] 
\geq {bigbnd}}{bu[j]  >=  bigbnd}. To specify the \eqn{{j}}{j}th constraint 
as an \emph{equality}, set \eqn{bu[{j}] = bl[{j}] = {\beta}}{bu[j] = bl[j] = \beta}, 
say, where \eqn{\mathrm{abs}({\beta}) < {bigbnd}}{abs(\beta) < bigbnd}. If \eqn{iobj 
= { - {1}}}{iobj =  - 1}, set \eqn{bu[n+\mathrm{abs}\left(iobj\right)] 
\geq {bigbnd}}{bu[n+abs(iobj)]  >=  bigbnd}.


}
\item{start}{string

Indicates how a starting basis is to be obtained. 

\eqn{start=\mathtt{'C'}}{start='C'}: An internal Crash procedure will be 
used to choose an initial basis.

\eqn{start=\mathtt{'W'}}{start='W'}: A basis is already defined in istate 
and ns (probably from a previous call).




}
\item{names}{string array

Specifies the column and row names to be used in the printed output.


}
\item{ns}{integer

\eqn{{n}_{{S}}}{n_S}, the number of superbasics. It need not be specified 
if \eqn{start=\mathtt{'C'}}{start='C'}, but must retain its value from 
a previous call when \eqn{start=\mathtt{'W'}}{start='W'}.


}
\item{xs}{double array

The initial values of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}. 
(See the description for istate.)


}
\item{istate}{integer array

If \eqn{start=\mathtt{'C'}}{start='C'}, the first n elements of istate 
and xs must specify the initial states and values, respectively, of the 
variables \eqn{{x}}{x}. (The slacks \eqn{{s}}{s} need not be initialized.) 
An internal Crash procedure is then used to select an initial basis matrix 
\eqn{{B}}{B}. The initial basis matrix will be triangular (neglecting certain 
small elements in each column). It is chosen from various rows and columns 
of \eqn{\left(\begin{array}{cc} {A}&-{I}\\ \end{array}\right)}{
(
A -I
)
}. Possible values for \eqn{istate[{j}]}{istate[j]} are as follows:


}
\item{clamda}{double array

If \eqn{ncnln > {0}}{ncnln > 0}, \eqn{clamda[{j}]}{clamda[j]} must contain 
a Lagrange multiplier estimate for the \eqn{{j}}{j}th nonlinear constraint 
\eqn{{F}_{{j}}\left({x}\right)}{F_j(x)} for \eqn{{j}=n+{1} \ldots n+ncnln}{j=n+1  . . .  n+ncnln}. 
If nothing special is known about the problem, or there is no wish to provide 
special information, you may set \eqn{clamda[{j}] = {0.0}}{clamda[j] = 0.0}. 
The remaining elements need not be set.


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Central Difference Interval} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt[{3}]{functionprecision}}{ = sqrt[3](functionprecision)} \cr
\code{Check Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {60}}{ = 60} \cr
\code{Crash Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}\mbox{ or }{3}}{ = 0 or 3} \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.1}}{ = 0.1} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Derivative Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {3}}{ = 3} \cr
\code{Derivative Linesearch} \tab \emph{} \tab Default \cr
\code{Nonderivative Linesearch} \tab \emph{} \tab  \cr
\code{Elastic Weight} \tab \emph{double} \tab Default \eqn{\mbox{} = {1.0}}{ = 1.0} or \eqn{{100.0}}{100.0} \cr
\code{Expand Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {10000}}{ = 10000} \cr
\code{Factorization Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {50}\mbox{ or }{100}}{ = 50 or 100} \cr
\code{Infeasible Exit} \tab \emph{} \tab Default \cr
\code{Feasible Exit} \tab \emph{} \tab  \cr
\code{Minimize} \tab \emph{} \tab Default \cr
\code{Maximize} \tab \emph{} \tab  \cr
\code{Feasible Point} \tab \emph{} \tab  \cr
\code{Forward Difference Interval} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{functionprecision}}{ = sqrt(functionprecision)} \cr
\code{Function Precision} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.8}}}{ = \epsilon^0.8} \cr
\code{Hessian Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {99999999}}{ = 99999999} \cr
\code{Hessian Full Memory} \tab \emph{} \tab Default when \eqn{\bar{{n}} < {75}}{barn < 75} \cr
\code{Hessian Limited Memory} \tab \emph{} \tab Default when \eqn{\bar{{n}} \geq {75}}{barn  >=  75} \cr
\code{Hessian Updates} \tab \emph{integer} \tab Default \eqn{\mbox{} = {20} or {99999999}}{ = 20 or 99999999} \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {10000}}{ = 10000} \cr
\code{Linesearch Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{List} \tab \emph{} \tab Default for \eqn{e04ug = list}{e04ug = list} \cr
\code{Nolist} \tab \emph{} \tab Default for \eqn{e04ug = nolist}{e04ug = nolist} \cr
\code{LU Density Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.6}}{ = 0.6} \cr
\code{LU Singularity Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.67}}}{ = \epsilon^0.67} \cr
\code{LU Factor Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {5.0}}{ = 5.0} or \eqn{{100.0}}{100.0} \cr
\code{LU Update Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {5.0}}{ = 5.0} or \eqn{{10.0}}{10.0} \cr
\code{Major Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Major Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1000}}{ = 1000} \cr
\code{Major Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Optimality Tolerance} \tab \emph{double} \tab  \cr
\code{Major Print Level} \tab \emph{integer} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Print Level} \tab \emph{} \tab  \cr
\code{Major Step Limit} \tab \emph{double} \tab Default \eqn{\mbox{} = {2.0}}{ = 2.0} \cr
\code{Minor Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab  \cr
\code{Minor Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {500}}{ = 500} \cr
\code{Minor Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Minor Print Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Monitoring File} \tab \emph{integer} \tab Default \eqn{\mbox{} = { - {1}}}{ =  - 1} \cr
\code{Partial Price} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1} or {10}}{ = 1 or 10} \cr
\code{Pivot Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.67}}}{ = \epsilon^0.67} \cr
\code{Scale Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1} or {2}}{ = 1 or 2} \cr
\code{Scale Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{Start Objective Check At Column} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Objective Check At Column} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}_{{1}}'}{ = n_1'} \cr
\code{Start Constraint Check At Column} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Constraint Check At Column} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}_{{1}}''}{ = n_1''} \cr
\code{Superbasics Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{min}({500}, {\bar{{n}} + {1}})}{ = min(500, barn + 1)} \cr
\code{Unbounded Objective} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{15}}}{ = 10^15} \cr
\code{Unbounded Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}\left({bigbnd} , {10}^{{20}}\right)}{ = max(bigbnd , 10^20)} \cr
\code{Verify Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Violation Limit} \tab \emph{double} \tab Default \eqn{\mbox{} = {10.0}}{ = 10.0} \cr
}
}
\item{nnz}{integer: \strong{default} = nrow(a)

The number of nonzero elements in \eqn{{A}}{A} (including the Jacobian 
for any nonlinear constraints). If \eqn{iobj = { - {1}}}{iobj =  - 1}, 
set \eqn{nnz = {1}}{nnz = 1}.


}
\item{nname}{integer: \strong{default} = nrow(names)

The number of column (i.e., variable) and row (i.e., constraint) names 
supplied in names. 

\eqn{nname = {1}}{nname = 1}: There are no names. Default names will be 
used in the printed output.

\eqn{nname = n + m}{nname = n + m}: All names must be supplied.




}
\item{leniz}{integer: \strong{default} = (max(500,(n+m)))




}{integer: \strong{default} = (max(500,(n+m)))




}
\item{lenz}{integer: \strong{default} = (500)




}{integer: \strong{default} = (500)




}
}

\details{
R interface to the NAG Fortran routine E04UGF.}

\value{

\item{A}{double array

Elements in the nonlinear part corresponding to nonlinear Jacobian variables 
are overwritten.


}
\item{NS}{integer

The final number of superbasics.


}
\item{XS}{double array

The final values of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}.


}
\item{ISTATE}{integer array

The final states of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}. 
The significance of each possible value of \eqn{istate[{j}]}{istate[j]} 
is as follows:


}
\item{CLAMDA}{double array

A set of Lagrange multipliers for the bounds on the variables (\emph{reduced 
costs}) and the general constraints (\emph{shadow costs}). More precisely, 
the first n elements contain the multipliers for the bounds on the variables, 
the next ncnln elements contain the multipliers for the nonlinear constraints 
\eqn{{F}\left({x}\right)}{F(x)} (if any) and the next (\eqn{m - ncnln}{m - ncnln}) 
elements contain the multipliers for the linear constraints \eqn{{G}{x}}{Gx} 
and the free row (if any).


}
\item{MINIZ}{integer

The minimum value of leniz required to start solving the problem. If \eqn{\mathrm{ifail} 
=12}{ifail =12}, e04ug may be called again with leniz suitably larger 
than miniz. (The bigger the better, since it is not certain how much workspace 
the basis factors need.)


}
\item{MINZ}{integer

The minimum value of lenz required to start solving the problem. If \eqn{\mathrm{ifail} 
=13}{ifail =13}, e04ug may be called again with lenz suitably larger than 
minz. (The bigger the better, since it is not certain how much workspace 
the basis factors need.)


}
\item{NINF}{integer

The number of constraints that lie outside their bounds by more than the 
value of the optional argument minorfeasibilitytolerance.


}
\item{SINF}{double

The sum of the infeasibilities of constraints that lie outside their bounds 
by more than the value of the optional argument minorfeasibilitytolerance.


}
\item{OBJ}{double

The value of the objective function.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04ugf.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
confun = function(mode, ncnln, njnln, nnzjac, x, fjac, 
    nstate) {
    
    f <- as.matrix(mat.or.vec(ncnln, 1))
    
    if (mode == 0 || mode == 2) {
        
        f[1] <- 1000 \%*\% sin(-x[1] - 0.25) + 1000 \%*\% sin(-x[2] - 
            0.25)
        
        f[2] <- 1000 \%*\% sin(x[1] - 0.25) + 1000 \%*\% sin(x[1] - 
            x[2] - 0.25)
        
        f[3] <- 1000 \%*\% sin(x[2] - x[1] - 0.25) + 1000 \%*\% sin(x[2] - 
            0.25)
        
    }
    
    if (mode == 1 || mode == 2) {
        
        fjac[1] <- -1000 \%*\% cos(-x[1] - 0.25)
        
        fjac[2] <- 1000 \%*\% cos(x[1] - 0.25) + 1000 \%*\% cos(x[1] - 
            x[2] - 0.25)
        
        fjac[3] <- -1000 \%*\% cos(x[2] - x[1] - 0.25)
        
        fjac[4] <- -1000 \%*\% cos(-x[2] - 0.25)
        
        fjac[5] <- -1000 \%*\% cos(x[1] - x[2] - 0.25)
        
        fjac[6] <- 1000 \%*\% cos(x[2] - x[1] - 0.25) + 1000 \%*\% 
            cos(x[2] - 0.25)
        
    }
    list(MODE = as.integer(mode), F = as.matrix(f), FJAC = as.matrix(fjac))
}
objfun = function(mode, nonln, x, objgrd, nstate) {
    
    
    if (mode == 0 || mode == 2) {
        
        objf <- 1e-06 \%*\% x[3]^3 + 2e-06 \%*\% x[4]^3/3
        
    }
    
    if (mode == 1 || mode == 2) {
        
        objgrd[1] <- 0
        
        objgrd[2] <- 0
        
        objgrd[3] <- 3e-06 \%*\% x[3]^2
        
        objgrd[4] <- 2e-06 \%*\% x[4]^2
        
    }
    list(MODE = as.integer(mode), OBJF = objf, OBJGRD = as.matrix(objgrd))
}

n <- 4

m <- 6

ncnln <- 3

nonln <- 4

njnln <- 2

iobj <- 6

a <- matrix(c(1e+25, 1e+25, 1e+25, 1, -1, 1e+25, 1e+25, 
    1e+25, -1, 1, 3, -1, -1, 2), nrow = 14, ncol = 1, byrow = TRUE)



ha <- matrix(c(1, 2, 3, 5, 4, 1, 2, 3, 5, 4, 6, 1, 
    2, 6), nrow = 14, ncol = 1, byrow = TRUE)



ka <- matrix(c(1, 6, 11, 13, 15), nrow = 5, ncol = 1, 
    byrow = TRUE)



bl <- matrix(c(-0.55, -0.55, 0, 0, -894.8, -894.8, 
    -1294.8, -0.55, -0.55, -1e+25), nrow = 10, ncol = 1, byrow = TRUE)



bu <- matrix(c(0.55, 0.55, 1200, 1200, -894.8, -894.8, 
    -1294.8, 1e+25, 1e+25, 1e+25), nrow = 10, ncol = 1, byrow = TRUE)



start <- "C"

names <- matrix(c("Varble 1", "Varble 2", "Varble 3", 
    "Varble 4", "NlnCon 1", "NlnCon 2", "NlnCon 3", "LinCon 1", 
    "LinCon 2", "Free Row"), nrow = 10, byrow = TRUE)



ns <- 0

xs <- matrix(c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0), nrow = 10, 
    ncol = 1, byrow = TRUE)



istate <- as.matrix(mat.or.vec(10, 1))

clamda <- matrix(c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0), 
    nrow = 10, ncol = 1, byrow = TRUE)



leniz <- 1000

lenz <- 1000

e04ug(confun, objfun, n, m, ncnln, nonln, njnln, 
    iobj, a, ha, ka, bl, bu, start, names, ns, xs, istate, clamda, 
    optlist)
}

\keyword{math}

\keyword{optimize}
\concept{NLP}
