
\name{g02ae}
\alias{g02ae}

\title{g02ae: Computes the nearest correlation matrix with k-factor structure to a real 
square matrix}

\description{
g02ae computes the factor loading matrix associated with the nearest correlation 
matrix with \eqn{{k}}{k}-factor structure, in the Frobenius norm, to a 
given square, input matrix.

}

\usage{
g02ae(g, k,
      n = nrow(g),
      errtol = 0.0,
      maxit = 0)
}

\arguments{

\item{g}{double array

\eqn{{G}}{G}, the initial matrix.


}
\item{k}{integer

\eqn{{k}}{k}, the number of factors and columns of \eqn{{X}}{X}.


}
\item{n}{integer: \strong{default} = nrow(g)

\eqn{{n}}{n}, the size of the matrix \eqn{{G}}{G}.


}
\item{errtol}{double: \strong{default} = 0.0

The termination tolerance for the projected gradient norm. See references 
for further details. If \eqn{errtol \leq {0.0}}{errtol  <=  0.0} then \eqn{{0.01}}{0.01} 
is used. This is often a suitable default value.


}
\item{maxit}{integer: \strong{default} = 0

Specifies the maximum number of iterations in the spectral projected gradient 
method.


}
}

\details{
R interface to the NAG Fortran routine G02AEF.}

\value{

\item{G}{double array

A symmetric matrix \eqn{\frac{{1}}{{2}}\left({G} + {G}^{T}\right)}{(1)/(2)(G + G^T)} 
with the diagonal elements set to unity.


}
\item{X}{double array

Contains the matrix \eqn{{X}}{X}.


}
\item{ITER}{integer

The number of steps taken in the spectral projected gradient method.


}
\item{FEVAL}{integer

The number of function evaluations.


}
\item{NRMPGD}{double

The norm of the projected gradient at the final iteration.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/G02/g02aef.pdf}
}

\author{NAG}

\examples{

ifail <- 0

errtol <- 1e-07

g <- matrix(c(2, -1, 0, 0, -1, 2, -1, 0, 0, -1, 2, 
    -1, 0, 0, -1, 2), nrow = 4, ncol = 4, byrow = TRUE)



k <- 2

maxits <- 200

maxit <- 10

ans <- g02ae(g, k)

if (ifail == 0) {
    
    writeLines(sprintf("\n Factor Loading Matrix x:\n", 
        "\n"))
    
    
    x <- ans$X
    
    print(x) 
    
    iter <- ans$ITER
    
    writeLines(sprintf("\n Number of Newton steps taken: \%d\n", 
        iter))
    
    
    feval <- ans$FEVAL
    
    writeLines(sprintf(" Number of function evaluations: \%d\n", 
        feval))
    
    
} 

}

\keyword{math}
\concept{Correlation Matrix}
