
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04cbf_FUNCT(CONST Integer *N, CONST double XC[],
                                  double *FC, Integer *E04CBF_IUSER,
                                  double E04CBF_RUSER[]);
extern void NAG_CALL e04cbf_MONIT(CONST double *FMIN, CONST double *FMAX,
                                  CONST double SIM[], CONST Integer *N,
                                  CONST Integer *NCALL,
                                  CONST double *SERROR, CONST double *VRATIO,
                                  Integer *E04CBF_IUSER,
                                  double E04CBF_RUSER[]);

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04CBF_clean_up(double *X);

SEXP WE04CBF(SEXP N, SEXP X, SEXP TOLF, SEXP TOLX, SEXP FUNCT, SEXP MONIT,
             SEXP MAXCAL,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04CBF_IUSER[6];
  double  E04CBF_RUSER[1];
  SEXP    ans, names;

/*initialization for input variables*/
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  double  *X_doubleMatrix = 0;
  double  TOLF_double;
  if (nag_extractDouble(TOLF, &TOLF_double))
    {
      error("Double extraction error for %s", "TOLF");
    }
  double  TOLX_double;
  if (nag_extractDouble(TOLX, &TOLX_double))
    {
      error("Double extraction error for %s", "TOLX");
    }
  int     MAXCAL_integer;
  if (nag_extractInteger(MAXCAL, &MAXCAL_integer))
    {
      error("Integer extraction error for %s", "MAXCAL");
    }

/*initialization for output variables*/
  double  F_double;

/*allocation general variables*/
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04CBF_clean_up(X_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Get the maximum number of iterations, or use default of 100 if not supplied */
  if (MAXCAL_integer == NA_INTEGER) MAXCAL_integer = 100;
  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04CBF_IUSER[0]);
  nag_convert_p2i(FUNCT, &E04CBF_IUSER[2]);
  nag_convert_p2i(MONIT, &E04CBF_IUSER[4]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04cbf_(&N_integer, X_doubleMatrix, &F_double, &TOLF_double, &TOLX_double,
          e04cbf_FUNCT, e04cbf_MONIT, &MAXCAL_integer, E04CBF_IUSER,
          E04CBF_RUSER,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04CBF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 3));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_d2r(1, &F_double));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 3));
  SET_STRING_ELT(names, 0, mkChar("X"));
  SET_STRING_ELT(names, 1, mkChar("F"));
  SET_STRING_ELT(names, 2, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04CBF_clean_up(X_doubleMatrix);

  return(ans);
}

void NAG_CALL e04cbf_FUNCT(CONST Integer *N, CONST double XC[], double *FC,
                           Integer *E04CBF_IUSER,
                           double E04CBF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04CBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04CBF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang3(
            funct,
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleScalarFromList("FC", result, FC);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04cbf_MONIT(CONST double *FMIN, CONST double *FMAX,
                           CONST double SIM[], CONST Integer *N,
                           CONST Integer *NCALL,
                           CONST double *SERROR, CONST double *VRATIO,
                           Integer *E04CBF_IUSER,
                           double E04CBF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04CBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04CBF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang8(
            funct,
            nag_d2r(1, FMIN),
            nag_d2r(1, FMAX),
            nag_md2r(1, *N+1, *N, SIM),
            nag_fi2r(1, N),
            nag_fi2r(1, NCALL),
            nag_d2r(1, SERROR),
            nag_d2r(1, VRATIO)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */

/* Release R_fcall */
  UNPROTECT(2);
}

void E04CBF_clean_up(double *X)
{
  if (X) free(X);
  return;
}
