
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04kzf_FUNCT2(CONST Integer *N, CONST double XC[],
                                   double *FC, double GC[],
                                   Integer *E04KZF_IUSER,
                                   double E04KZF_RUSER[]);

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04KZF_clean_up(double *BL, double *BU, double *X, double *G, Integer *IW,
                     double *W);

SEXP WE04KZF(SEXP N, SEXP IBOUND, SEXP FUNCT2, SEXP BL, SEXP BU, SEXP X,
             SEXP rho)
{
  Integer IFAIL_integer;
  Integer E04KZF_IUSER[4];
  double  E04KZF_RUSER[1];
  SEXP    ans, names;

/*initialization for input variables*/
  Integer N_integer = INTEGER(N)[0];
  Integer IBOUND_integer = INTEGER(IBOUND)[0];
  double  *BL_doubleMatrix = 0;
  double  *BU_doubleMatrix = 0;
  double  *X_doubleMatrix = 0;

/*initialization for dimension variables*/
  Integer LIW_integer = N_integer+2;
  Integer LW_integer = MAX(N_integer*(N_integer+7), 10);

/*initialization for output variables*/
  double  F_double;
  double  *G_doubleMatrix = 0;

/*allocation general variables*/
  Integer BL_dim1 = N_integer;
  Integer BL_dim2 = 1;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer BU_dim1 = N_integer;
  Integer BU_dim2 = 1;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer X_dim1 = N_integer;
  Integer X_dim2 = 1;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer G_dim1 = N_integer;
  Integer G_dim2 = 1;
  if (!(G_doubleMatrix = (double *) malloc(G_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }

/*initialization for workspace variables*/
  Integer *IW_integerMatrix = 0;
  double  *W_doubleMatrix = 0;

/*allocation for workspace variables*/
  Integer IW_dim1 = LIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error");
    }
  Integer W_dim1 = LW_integer;
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if (nag_extractDoubleMatrix(1, BL_dim1, BL_dim2, BL, BL_doubleMatrix))
    {
      E04KZF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                      G_doubleMatrix, IW_integerMatrix,
                      W_doubleMatrix);
      error("Array extraction error for %s", "BL");
    }
  if (nag_extractDoubleMatrix(1, BU_dim1, BU_dim2, BU, BU_doubleMatrix))
    {
      E04KZF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                      G_doubleMatrix, IW_integerMatrix,
                      W_doubleMatrix);
      error("Array extraction error for %s", "BU");
    }
  if (nag_extractDoubleMatrix(1, X_dim1, X_dim2, X, X_doubleMatrix))
    {
      E04KZF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                      G_doubleMatrix, IW_integerMatrix,
                      W_doubleMatrix);
      error("Array extraction error for %s", "X");
    }

/* Perform some initial checks */
  if (!isFunction(FUNCT2))
    {
      error("FUNCT2 must be a function");
    }
  if (!isEnvironment(rho))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(rho, &E04KZF_IUSER[0]);
  nag_convert_p2i(FUNCT2, &E04KZF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04kzf_(&N_integer, &IBOUND_integer, e04kzf_FUNCT2, BL_doubleMatrix,
          BU_doubleMatrix, X_doubleMatrix, &F_double, G_doubleMatrix,
          IW_integerMatrix,
          &LIW_integer, W_doubleMatrix, &LW_integer, E04KZF_IUSER,
          E04KZF_RUSER,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      if (IFAIL_integer == 5)
        {
          warning(
            "WARNING on EXIT from NAG library routine  E04KZF, : IFAIL = %ld\n",
            (long) IFAIL_integer);
        }
      else
        {
          warning(
            "ABNORMAL EXIT from NAG library routine  E04KZF, : IFAIL = %ld\n",
            (long) IFAIL_integer);
        }
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 6));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, BL_dim1, BL_dim2, BL_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, BU_dim1, BU_dim2, BU_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, X_dim1, X_dim2, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_d2r(1, &F_double));
  SET_VECTOR_ELT(ans, 4, nag_md2r(1, G_dim1, G_dim2, G_doubleMatrix));
  SET_VECTOR_ELT(ans, 5, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 6));
  SET_STRING_ELT(names, 0, mkChar("BL"));
  SET_STRING_ELT(names, 1, mkChar("BU"));
  SET_STRING_ELT(names, 2, mkChar("X"));
  SET_STRING_ELT(names, 3, mkChar("F"));
  SET_STRING_ELT(names, 4, mkChar("G"));
  SET_STRING_ELT(names, 5, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04KZF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                  G_doubleMatrix, IW_integerMatrix,
                  W_doubleMatrix);

  return(ans);
}

void NAG_CALL e04kzf_FUNCT2(CONST Integer *N, CONST double XC[], double *FC,
                            double GC[], Integer *E04KZF_IUSER,
                            double E04KZF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04KZF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04KZF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang5(
            funct,
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_d2r(1, FC),
            nag_md2r(1, *N, 1, GC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleScalarFromList("FC", result, FC);
  (void) nag_getDoubleMatrixFromList("GC", 1, *N, 1, result, GC);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04KZF_clean_up(double *BL, double *BU, double *X, double *G, Integer *IW,
                     double *W)
{
  if (BL) free(BL);
  if (BU) free(BU);
  if (X) free(X);
  if (G) free(G);
  if (IW) free(IW);
  if (W) free(W);
  return;
}
