
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04wdf_CONFUN(Integer *MODE, CONST Integer *NCNLN,
                                   CONST Integer *N, CONST Integer *LDCJ,
                                   CONST Integer NEEDC[], CONST double X[],
                                   double CCON[], double CJAC[],
                                   CONST Integer *NSTATE,
                                   Integer *E04WDF_IUSER,
                                   double E04WDF_RUSER[]);
extern void NAG_CALL e04wdf_OBJFUN(Integer *MODE, CONST Integer *N,
                                   CONST double X[], double *OBJF,
                                   double GRAD[], CONST Integer *NSTATE,
                                   Integer *E04WDF_IUSER,
                                   double E04WDF_RUSER[]);

#ifdef __cplusplus
}
#endif
int wE04WDF_handle_options(SEXP OPTLIST, Integer IW_integerMatrix[],
                           double RW_doubleMatrix[])
;

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04WDF_clean_up(double *A, double *BL, double *BU, Integer *ISTATE,
                     double *CCON, double *CJAC, double *CLAMDA, double *GRAD,
                     double *H, double *X,
                     Integer *IW,
                     double *RW);

SEXP WE04WDF(SEXP N, SEXP NCLIN, SEXP NCNLN, SEXP A, SEXP BL, SEXP BU,
             SEXP CONFUN, SEXP OBJFUN, SEXP ISTATE, SEXP CCON, SEXP CJAC,
             SEXP CLAMDA,
             SEXP H, SEXP X, SEXP OPTLIST,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04WDF_IUSER[6];
  double  E04WDF_RUSER[1];
  SEXP    ans, names;
  Integer nadv, iflag;

  /* Initialize the option arrays */
  Integer IFAIL_initializer_integer = 1;
  char    err_msg[LEN_ERR_MSG];

/*Declare dimension variables for option initializer*/
  Integer LENIW_integer = 600;
  Integer LENRW_integer = 600;

/*Declare the parameters for the option initializer*/
  Integer *IW_integerMatrix = 0;
  double  *RW_doubleMatrix = 0;
  Integer IW_dim1 = LENIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer RW_dim1 = LENRW_integer;
  if (!(RW_doubleMatrix = (double *) malloc(RW_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "RW");
    }

  nag_initializeIntegerMatrix(IW_dim1, 1, IW_integerMatrix);
  nag_initializeDoubleMatrix(RW_dim1, 1, RW_doubleMatrix);
  e04wcf_(IW_integerMatrix, &LENIW_integer, RW_doubleMatrix, &LENRW_integer,
          &IFAIL_initializer_integer);

  if (IFAIL_initializer_integer != 0)
    {
      sprintf(err_msg, "E04WCF failed with IFAIL = %ld\n",
              (long) IFAIL_initializer_integer);
      error(err_msg);
    } /* Handle options */
  int     unit_num;

  unit_num = wE04WDF_handle_options(OPTLIST, IW_integerMatrix, RW_doubleMatrix);
  if (unit_num < 0)
    {
      /* Error opening output file */
      sprintf(
        err_msg,
        "Error opening monitoring file for output, check value for `monitoring file` entry in `optlist`");
      error(err_msg);
    }
  else if (unit_num > 0)
    {
      /* We are using an output file, so need to handle the advisory channel */

      /* Remember the current value */
      iflag = 0;
      x04abf_(&iflag, &nadv);

      /* Set the advisory channel to the output file. Need to do this, otherwise E04WDF sends output to both the
         advisory channel and the output file */
      iflag = 1;
      x04abf_(&iflag, &unit_num);
    }

/*initialization for input variables*/
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int     NCLIN_integer;
  if (nag_extractInteger(NCLIN, &NCLIN_integer))
    {
      error("Integer extraction error for %s", "NCLIN");
    }
  int     NCNLN_integer;
  if (nag_extractInteger(NCNLN, &NCNLN_integer))
    {
      error("Integer extraction error for %s", "NCNLN");
    }
  double  *A_doubleMatrix = 0;
  double  *BL_doubleMatrix = 0;
  double  *BU_doubleMatrix = 0;
  Integer *ISTATE_integerMatrix = 0;
  double  *CCON_doubleMatrix = 0;
  double  *CJAC_doubleMatrix = 0;
  double  *CLAMDA_doubleMatrix = 0;
  double  *H_doubleMatrix = 0;
  double  *X_doubleMatrix = 0;

/*initialization for dimension variables*/
  Integer LDA_integer = nrows(A);
  Integer LDCJ_integer = MAX(1, NCNLN_integer);
  Integer LDH_integer = N_integer;
/*initialization for output variables*/
  int     MAJITS_integer;
  double  OBJF_double;
  double  *GRAD_doubleMatrix = 0;

/*allocation general variables*/
  Integer A_dim1 = LDA_integer;
  Integer A_dim2;
  if ((NCLIN_integer > 0))
    {
      A_dim2 = N_integer;
    }
  else
    {
      A_dim2 = 1;
    }
  if (!(A_doubleMatrix = (double *) malloc(A_dim1*A_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "A");
    }
  Integer BL_dim1 = N_integer+NCLIN_integer+NCNLN_integer;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BL");
    }
  Integer BU_dim1 = N_integer+NCLIN_integer+NCNLN_integer;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BU");
    }
  Integer ISTATE_dim1 = N_integer+NCLIN_integer+NCNLN_integer;
  if (!(ISTATE_integerMatrix = (Integer *) malloc(ISTATE_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "ISTATE");
    }
  Integer CCON_dim1 = MAX(1, NCNLN_integer);
  if (!(CCON_doubleMatrix = (double *) malloc(CCON_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "CCON");
    }
  Integer CJAC_dim1 = LDCJ_integer;
  Integer CJAC_dim2;
  if (NCNLN_integer > 0)
    {
      CJAC_dim2 = N_integer;
    }
  else
    {
      CJAC_dim2 = 1;
    }
  if (!(CJAC_doubleMatrix =
          (double *) malloc(CJAC_dim1*CJAC_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "CJAC");
    }
  Integer CLAMDA_dim1 = N_integer+NCLIN_integer+NCNLN_integer;
  if (!(CLAMDA_doubleMatrix = (double *) malloc(CLAMDA_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "CLAMDA");
    }
  Integer GRAD_dim1 = N_integer;
  if (!(GRAD_doubleMatrix = (double *) malloc(GRAD_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "GRAD");
    }
  Integer H_dim1 = LDH_integer;
  Integer H_dim2 = N_integer;
  if (!(H_doubleMatrix = (double *) malloc(H_dim1*H_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "H");
    }
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, A_dim1, A_dim2, A, A_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "A", nrows(A), A_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "A");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BL_dim1, 1, BL, BL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BL", nrows(BL), BL_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "BL");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BU_dim1, 1, BU, BU_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BU", nrows(BU), BU_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "BU");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, ISTATE_dim1, 1, ISTATE,
                                  ISTATE_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "ISTATE", nrows(ISTATE), ISTATE_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "ISTATE");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, CCON_dim1, 1, CCON, CCON_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "CCON", nrows(CCON), CCON_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "CCON");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, CJAC_dim1, CJAC_dim2, CJAC,
                                 CJAC_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "CJAC", nrows(CJAC), CJAC_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "CJAC");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, CLAMDA_dim1, 1, CLAMDA,
                                 CLAMDA_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "CLAMDA", nrows(CLAMDA), CLAMDA_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "CLAMDA");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, H_dim1, H_dim2, H, H_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "H", nrows(H), H_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "H");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          ISTATE_integerMatrix, CCON_doubleMatrix,
                          CJAC_doubleMatrix,
                          CLAMDA_doubleMatrix, GRAD_doubleMatrix,
                          H_doubleMatrix, X_doubleMatrix,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04WDF_IUSER[0]);
  nag_convert_p2i(CONFUN, &E04WDF_IUSER[2]);
  nag_convert_p2i(OBJFUN, &E04WDF_IUSER[4]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04wdf_(&N_integer, &NCLIN_integer, &NCNLN_integer, &LDA_integer,
          &LDCJ_integer, &LDH_integer, A_doubleMatrix, BL_doubleMatrix,
          BU_doubleMatrix,
          e04wdf_CONFUN, e04wdf_OBJFUN, &MAJITS_integer, ISTATE_integerMatrix,
          CCON_doubleMatrix, CJAC_doubleMatrix, CLAMDA_doubleMatrix,
          &OBJF_double,
          GRAD_doubleMatrix, H_doubleMatrix, X_doubleMatrix, IW_integerMatrix,
          &LENIW_integer, RW_doubleMatrix, &LENRW_integer, E04WDF_IUSER,
          E04WDF_RUSER,
          &IFAIL_integer);

  /* Close the monitoring file if one opened */

  if (unit_num > 0)
    {
      IFAIL_integer = 1;
      x04adf_(&unit_num, &IFAIL_integer);

      /* Set the advisory channel back to its previous value */
      iflag = 1;
      x04abf_(&iflag, &nadv);
    }

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04WDF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 10));
  SET_VECTOR_ELT(ans, 0, nag_i2r(1, &MAJITS_integer));
  SET_VECTOR_ELT(ans, 1, nag_mi2r(1, ISTATE_dim1, 1, ISTATE_integerMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, CCON_dim1, 1, CCON_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_md2r(1, CJAC_dim1, CJAC_dim2, CJAC_doubleMatrix));
  SET_VECTOR_ELT(ans, 4, nag_md2r(1, CLAMDA_dim1, 1, CLAMDA_doubleMatrix));
  SET_VECTOR_ELT(ans, 5, nag_d2r(1, &OBJF_double));
  SET_VECTOR_ELT(ans, 6, nag_md2r(1, GRAD_dim1, 1, GRAD_doubleMatrix));
  SET_VECTOR_ELT(ans, 7, nag_md2r(1, H_dim1, H_dim2, H_doubleMatrix));
  SET_VECTOR_ELT(ans, 8, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 9, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 10));
  SET_STRING_ELT(names, 0, mkChar("MAJITS"));
  SET_STRING_ELT(names, 1, mkChar("ISTATE"));
  SET_STRING_ELT(names, 2, mkChar("CCON"));
  SET_STRING_ELT(names, 3, mkChar("CJAC"));
  SET_STRING_ELT(names, 4, mkChar("CLAMDA"));
  SET_STRING_ELT(names, 5, mkChar("OBJF"));
  SET_STRING_ELT(names, 6, mkChar("GRAD"));
  SET_STRING_ELT(names, 7, mkChar("H"));
  SET_STRING_ELT(names, 8, mkChar("X"));
  SET_STRING_ELT(names, 9, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04WDF_clean_up(A_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix,
                  ISTATE_integerMatrix, CCON_doubleMatrix, CJAC_doubleMatrix,
                  CLAMDA_doubleMatrix, GRAD_doubleMatrix, H_doubleMatrix,
                  X_doubleMatrix,
                  IW_integerMatrix,
                  RW_doubleMatrix);

  return(ans);
}

void NAG_CALL e04wdf_CONFUN(Integer *MODE, CONST Integer *NCNLN,
                            CONST Integer *N, CONST Integer *LDCJ,
                            CONST Integer NEEDC[], CONST double X[],
                            double CCON[], double CJAC[],
                            CONST Integer *NSTATE, Integer *E04WDF_IUSER,
                            double E04WDF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04WDF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04WDF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang8(
            funct,
            nag_fi2r(1, MODE),
            nag_fi2r(1, NCNLN),
            nag_fi2r(1, N),
            nag_fi2r(*NCNLN, NEEDC),
            nag_md2r(1, *N, 1, X),
            nag_md2r(1, *LDCJ, *N, CJAC),
            nag_fi2r(1, NSTATE)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("MODE", result, MODE);
  (void) nag_getDoubleMatrixFromList("CCON", 1, MAX(1,
                                                    *NCNLN), 1, result, CCON);
  (void) nag_getDoubleMatrixFromList("CJAC", 1, *LDCJ, *N, result, CJAC);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04wdf_OBJFUN(Integer *MODE, CONST Integer *N, CONST double X[],
                            double *OBJF, double GRAD[], CONST Integer *NSTATE,
                            Integer *E04WDF_IUSER,
                            double E04WDF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04WDF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04WDF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang6(
            funct,
            nag_fi2r(1, MODE),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, X),
            nag_md2r(1, *N, 1, GRAD),
            nag_fi2r(1, NSTATE)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("MODE", result, MODE);
  (void) nag_getDoubleScalarFromList("OBJF", result, OBJF);
  (void) nag_getDoubleMatrixFromList("GRAD", 1, *N, 1, result, GRAD);

/* Release R_fcall */
  UNPROTECT(2);
}

int wE04WDF_handle_options(SEXP OPTLIST, Integer IW_integerMatrix[],
                           double RW_doubleMatrix[])
{
  SEXP       names, tentry;
  char       err_msg[LEN_ERR_MSG];
  int        i, ival;
  Integer    IFAIL_setter_integer, STRING_length;
  double     dval;
  const char *this_name;
  char       STRING_char[100];
  int        unit_num = 0;

  /* No options specified */
  if (!isNewList(OPTLIST))
    {
      return 0;
    }

  /* Get the list of names from the list */
  PROTECT(names = getAttrib(OPTLIST, R_NamesSymbol));
  for (i = 0; i < length(OPTLIST); i++)
    {
      this_name = CHAR(STRING_ELT(names, i));

      /* Only interested in the first element, which is what asReal should
         convert to a double */
      PROTECT(tentry = VECTOR_ELT(OPTLIST, i));
      if (!isString(tentry))
        {
          dval = asReal(tentry);
          if (!R_FINITE(dval)) dval = 0.0;
          ival = (int) dval;
        }
      else
        {
          dval = -1.0;
          ival = -1;
        }

      STRING_char[0] = '\0';
      if (!strcmp(this_name, "list"))
        {
          if (ival)
            {
              sprintf(STRING_char, "list");
            }
          else
            {
              sprintf(STRING_char, "nolist");
            }
        }
      else if (!strcmp(this_name, "Central Difference Interval"))
        {
          sprintf(STRING_char, "Central Difference Interval = %g", dval);
        }
      else if (!strcmp(this_name, "Check Frequency"))
        {
          sprintf(STRING_char, "Check Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Option"))
        {
          sprintf(STRING_char, "Crash Option = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Tolerance"))
        {
          sprintf(STRING_char, "Crash Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Derivative Level"))
        {
          sprintf(STRING_char, "Derivative Level = %d", ival);
        }
      else if (!strcmp(this_name, "Difference Interval"))
        {
          sprintf(STRING_char, "Difference Interval = %g", dval);
        }
      else if (!strcmp(this_name, "Elastic Weight"))
        {
          sprintf(STRING_char, "Elastic Weight = %g", dval);
        }
      else if (!strcmp(this_name, "Expand Frequency"))
        {
          sprintf(STRING_char, "Expand Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Factorization Frequency"))
        {
          sprintf(STRING_char, "Factorization Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Function Precision"))
        {
          sprintf(STRING_char, "Function Precision = %g", dval);
        }
      else if (!strcmp(this_name, "Hessian Frequency"))
        {
          sprintf(STRING_char, "Hessian Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Hessian Updates"))
        {
          sprintf(STRING_char, "Hessian Updates = %d", ival);
        }
      else if (!strcmp(this_name, "Infinite Bound Size"))
        {
          sprintf(STRING_char, "Infinite Bound Size = %g", dval);
        }
      else if (!strcmp(this_name, "Iterations Limit"))
        {
          sprintf(STRING_char, "Iterations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Linesearch Tolerance"))
        {
          sprintf(STRING_char, "Linesearch Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Major Feasibility Tolerance"))
        {
          sprintf(STRING_char, "Major Feasibility Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Major Optimality Tolerance"))
        {
          sprintf(STRING_char, "Major Optimality Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Major Iterations Limit"))
        {
          sprintf(STRING_char, "Major Iterations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Major Print Level"))
        {
          sprintf(STRING_char, "Major Print Level = %d", ival);
        }
      else if (!strcmp(this_name, "Major Step Limit"))
        {
          sprintf(STRING_char, "Major Step Limit = %g", dval);
        }
      else if (!strcmp(this_name, "Feasibility Tolerance"))
        {
          sprintf(STRING_char, "Feasibility Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Minor Iterations Limit"))
        {
          sprintf(STRING_char, "Minor Iterations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Minor Print Level"))
        {
          sprintf(STRING_char, "Minor Print Level = %d", ival);
        }
      else if (!strcmp(this_name, "New Superbasics Limit"))
        {
          sprintf(STRING_char, "New Superbasics Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Old Basis File"))
        {
          sprintf(STRING_char, "Old Basis File = %d", ival);
        }
      else if (!strcmp(this_name, "Partial Price"))
        {
          sprintf(STRING_char, "Partial Price = %d", ival);
        }
      else if (!strcmp(this_name, "Pivot Tolerance"))
        {
          sprintf(STRING_char, "Pivot Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Print File"))
        {
          sprintf(STRING_char, "Print File = %d", ival);
        }
      else if (!strcmp(this_name, "Print Frequency"))
        {
          sprintf(STRING_char, "Print Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Proximal Point Method"))
        {
          sprintf(STRING_char, "Proximal Point Method = %d", ival);
        }
      else if (!strcmp(this_name, "Reduced Hessian Dimension"))
        {
          sprintf(STRING_char, "Reduced Hessian Dimension = %d", ival);
        }
      else if (!strcmp(this_name, "Scale Option"))
        {
          sprintf(STRING_char, "Scale Option = %d", ival);
        }
      else if (!strcmp(this_name, "Scale Tolerance"))
        {
          sprintf(STRING_char, "Scale Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Solution File"))
        {
          sprintf(STRING_char, "Solution File = %d", ival);
        }
      else if (!strcmp(this_name, "Start Objective Check At Variable"))
        {
          sprintf(STRING_char, "Start Objective Check At Variable = %d", ival);
        }
      else if (!strcmp(this_name, "Stop Objective Check At Variable"))
        {
          sprintf(STRING_char, "Stop Objective Check At Variable = %d", ival);
        }
      else if (!strcmp(this_name, "Start Constraint Check At Variable"))
        {
          sprintf(STRING_char, "Start Constraint Check At Variable = %d", ival);
        }
      else if (!strcmp(this_name, "Stop Constraint Check At Variable"))
        {
          sprintf(STRING_char, "Stop Constraint Check At Variable = %d", ival);
        }
      else if (!strcmp(this_name, "Superbasics Limit"))
        {
          sprintf(STRING_char, "Superbasics Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Timing Level"))
        {
          sprintf(STRING_char, "Timing Level = %d", ival);
        }
      else if (!strcmp(this_name, "Verify Level"))
        {
          sprintf(STRING_char, "Verify Level = %d", ival);
        }
      else if (!strcmp(this_name, "Violation Limit"))
        {
          sprintf(STRING_char, "Violation Limit = %g", dval);
        }

      /* All other options are ignored (this includes the valid ones: cold start and warm start) */

      UNPROTECT(1);

      /* Call the option setting routine */
      if (STRING_char[0])
        {
          IFAIL_setter_integer = 1;
          STRING_length = strlen(STRING_char);
          e04wff_(STRING_char, IW_integerMatrix, RW_doubleMatrix,
                  &IFAIL_setter_integer,
                  STRING_length);

          if (IFAIL_setter_integer != 0)
            {
              sprintf(err_msg, "E04WFF failed with IFAIL = %ld\n",
                      (long) IFAIL_setter_integer);
              error(err_msg);
            }
        }
    }
  UNPROTECT(1);
  return unit_num;
}

void E04WDF_clean_up(double *A, double *BL, double *BU, Integer *ISTATE,
                     double *CCON, double *CJAC, double *CLAMDA, double *GRAD,
                     double *H, double *X,
                     Integer *IW,
                     double *RW)
{
  if (A) free(A);
  if (BL) free(BL);
  if (BU) free(BU);
  if (ISTATE) free(ISTATE);
  if (CCON) free(CCON);
  if (CJAC) free(CJAC);
  if (CLAMDA) free(CLAMDA);
  if (GRAD) free(GRAD);
  if (H) free(H);
  if (X) free(X);
  if (IW) free(IW);
  if (RW) free(RW);
  return;
}
