
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04ybf_LSQFUN(Integer *IFLAG, CONST Integer *M,
                                   CONST Integer *N, CONST double XC[],
                                   double FVEC[], double FJAC[],
                                   CONST Integer *LDFJAC, Integer IW[],
                                   CONST Integer *LIW, double W[],
                                   CONST Integer *LW);
extern void NAG_CALL e04ybf_LSQHES(Integer *IFLAG, CONST Integer *M,
                                   CONST Integer *N, CONST double FVEC[],
                                   CONST double XC[], double B[],
                                   CONST Integer *LB, Integer IW[],
                                   CONST Integer *LIW, double W[],
                                   CONST Integer *LW);

#ifdef __cplusplus
}
#endif

/*Declaration of global IUSER for this routine*/
Integer E04YBF_IUSER[6];

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04YBF_clean_up(double *X, double *FVEC, double *FJAC, double *B,
                     Integer *IW,
                     double *W);

SEXP WE04YBF(SEXP M, SEXP N, SEXP LSQFUN, SEXP LSQHES, SEXP X, SEXP LB,
             SEXP IW, SEXP W,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  int     M_integer;
  if (nag_extractInteger(M, &M_integer))
    {
      error("Integer extraction error for %s", "M");
    }
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  double  *X_doubleMatrix = 0;
  int     LB_integer;
  if (nag_extractInteger(LB, &LB_integer))
    {
      error("Integer extraction error for %s", "LB");
    }
  Integer *IW_integerMatrix = 0;
  double  *W_doubleMatrix = 0;

/*initialization for dimension variables*/
  Integer LDFJAC_integer = M_integer;
  Integer LIW_integer = 1;
  Integer LW_integer;
  if (N_integer > 1)
    {
      LW_integer = 5*N_integer+M_integer+M_integer*N_integer+N_integer*
                   (N_integer-1) / 2;
    }
  else
    {
      if (N_integer == 1)
        {
          LW_integer = 6+2*M_integer;
        }
    }

/*initialization for output variables*/
  double  *FVEC_doubleMatrix = 0;
  double  *FJAC_doubleMatrix = 0;
  double  *B_doubleMatrix = 0;

/*allocation general variables*/
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer FVEC_dim1 = M_integer;
  if (!(FVEC_doubleMatrix = (double *) malloc(FVEC_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "FVEC");
    }
  Integer FJAC_dim1 = LDFJAC_integer;
  Integer FJAC_dim2 = N_integer;
  if (!(FJAC_doubleMatrix =
          (double *) malloc(FJAC_dim1*FJAC_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "FJAC");
    }
  Integer B_dim1 = LB_integer;
  if (!(B_doubleMatrix = (double *) malloc(B_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "B");
    }
  Integer IW_dim1 = LIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer W_dim1 = LW_integer;
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "W");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04YBF_clean_up(X_doubleMatrix, FVEC_doubleMatrix, FJAC_doubleMatrix,
                          B_doubleMatrix, IW_integerMatrix,
                          W_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04YBF_IUSER[0]);
  nag_convert_p2i(LSQFUN, &E04YBF_IUSER[2]);
  nag_convert_p2i(LSQHES, &E04YBF_IUSER[4]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04ybf_(&M_integer, &N_integer, e04ybf_LSQFUN, e04ybf_LSQHES, X_doubleMatrix,
          FVEC_doubleMatrix, FJAC_doubleMatrix, &LDFJAC_integer,
          B_doubleMatrix,
          &LB_integer, IW_integerMatrix, &LIW_integer, W_doubleMatrix,
          &LW_integer,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04YBF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 6));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, FVEC_dim1, 1, FVEC_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, FJAC_dim1, FJAC_dim2, FJAC_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, B_dim1, 1, B_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_mi2r(1, IW_dim1, 1, IW_integerMatrix));
  SET_VECTOR_ELT(ans, 4, nag_md2r(1, W_dim1, 1, W_doubleMatrix));
  SET_VECTOR_ELT(ans, 5, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 6));
  SET_STRING_ELT(names, 0, mkChar("FVEC"));
  SET_STRING_ELT(names, 1, mkChar("FJAC"));
  SET_STRING_ELT(names, 2, mkChar("B"));
  SET_STRING_ELT(names, 3, mkChar("IW"));
  SET_STRING_ELT(names, 4, mkChar("W"));
  SET_STRING_ELT(names, 5, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04YBF_clean_up(X_doubleMatrix, FVEC_doubleMatrix, FJAC_doubleMatrix,
                  B_doubleMatrix, IW_integerMatrix,
                  W_doubleMatrix);

  return(ans);
}

void NAG_CALL e04ybf_LSQFUN(Integer *IFLAG, CONST Integer *M, CONST Integer *N,
                            CONST double XC[], double FVEC[], double FJAC[],
                            CONST Integer *LDFJAC,
                            Integer IW[], CONST Integer *LIW, double W[],
                            CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04YBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04YBF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang6(
            funct,
            nag_fi2r(1, IFLAG),
            nag_fi2r(1, M),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_fi2r(1, LDFJAC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("IFLAG", result, IFLAG);
  (void) nag_getDoubleMatrixFromList("FVEC", 1, *M, 1, result, FVEC);
  (void) nag_getDoubleMatrixFromList("FJAC", 1, *LDFJAC, *N, result, FJAC);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04ybf_LSQHES(Integer *IFLAG, CONST Integer *M, CONST Integer *N,
                            CONST double FVEC[], CONST double XC[], double B[],
                            CONST Integer *LB,
                            Integer IW[], CONST Integer *LIW, double W[],
                            CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04YBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04YBF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang7(
            funct,
            nag_fi2r(1, IFLAG),
            nag_fi2r(1, M),
            nag_fi2r(1, N),
            nag_md2r(1, *M, 1, FVEC),
            nag_md2r(1, *N, 1, XC),
            nag_fi2r(1, LB)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("IFLAG", result, IFLAG);
  (void) nag_getDoubleMatrixFromList("B", 1, *LB, 1, result, B);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04YBF_clean_up(double *X, double *FVEC, double *FJAC, double *B,
                     Integer *IW,
                     double *W)
{
  if (X) free(X);
  if (FVEC) free(FVEC);
  if (FJAC) free(FJAC);
  if (B) free(B);
  if (IW) free(IW);
  if (W) free(W);
  return;
}
