
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04YCF_clean_up(double *S, double *V, double *CJ, double *WORK);

SEXP WE04YCF(SEXP JOB, SEXP M, SEXP N, SEXP FSUMSQ, SEXP S, SEXP V)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  int     JOB_integer;
  if (nag_extractInteger(JOB, &JOB_integer))
    {
      error("Integer extraction error for %s", "JOB");
    }
  int     M_integer;
  if (nag_extractInteger(M, &M_integer))
    {
      error("Integer extraction error for %s", "M");
    }
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  double  FSUMSQ_double;
  if (nag_extractDouble(FSUMSQ, &FSUMSQ_double))
    {
      error("Double extraction error for %s", "FSUMSQ");
    }
  double  *S_doubleMatrix = 0;
  double  *V_doubleMatrix = 0;

/*initialization for dimension variables*/
  Integer LDV_integer;
  LDV_integer = N_integer;

/*initialization for output variables*/
  double  *CJ_doubleMatrix = 0;

/*allocation general variables*/
  Integer S_dim1 = N_integer;
  if (!(S_doubleMatrix = (double *) malloc(S_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "S");
    }
  Integer V_dim1 = LDV_integer;
  Integer V_dim2 = N_integer;
  if (!(V_doubleMatrix = (double *) malloc(V_dim1*V_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "V");
    }
  Integer CJ_dim1 = N_integer;
  if (!(CJ_doubleMatrix = (double *) malloc(CJ_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "CJ");
    }

/*initialization for workspace variables*/
  double  *WORK_doubleMatrix = 0;

/*allocation for workspace variables*/
  Integer WORK_dim1 = N_integer;
  if (!(WORK_doubleMatrix = (double *) malloc(WORK_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "WORK");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, S_dim1, 1, S, S_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "S", nrows(S), S_dim1);
        }
      else
        {
          E04YCF_clean_up(S_doubleMatrix, V_doubleMatrix, CJ_doubleMatrix,
                          WORK_doubleMatrix);
          error("Array extraction error for %s", "S");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, V_dim1, V_dim2, V, V_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "V", nrows(V), V_dim1);
        }
      else
        {
          E04YCF_clean_up(S_doubleMatrix, V_doubleMatrix, CJ_doubleMatrix,
                          WORK_doubleMatrix);
          error("Array extraction error for %s", "V");
        }
    }

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04ycf_(&JOB_integer, &M_integer, &N_integer, &FSUMSQ_double, S_doubleMatrix,
          V_doubleMatrix, &LDV_integer, CJ_doubleMatrix, WORK_doubleMatrix,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04YCF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 3));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, V_dim1, V_dim2, V_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, CJ_dim1, 1, CJ_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 3));
  SET_STRING_ELT(names, 0, mkChar("V"));
  SET_STRING_ELT(names, 1, mkChar("CJ"));
  SET_STRING_ELT(names, 2, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04YCF_clean_up(S_doubleMatrix, V_doubleMatrix, CJ_doubleMatrix,
                  WORK_doubleMatrix);

  return(ans);
}

void E04YCF_clean_up(double *S, double *V, double *CJ, double *WORK)
{
  if (S) free(S);
  if (V) free(V);
  if (CJ) free(CJ);
  if (WORK) free(WORK);
  return;
}
