
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void F08FAF_clean_up(double *A, double *W, double *WORK);

SEXP WF08FAF(SEXP JOBZ, SEXP UPLO, SEXP N, SEXP A)
{
  Integer    IFAIL_integer, IFAIL_EXTRACTION_integer;
  SEXP       ans, names;

/*initialization for input variables*/
  const char *JOBZ_char = CHAR(STRING_ELT(JOBZ, 0));
  Integer    JOBZ_length = 1;
  const char *UPLO_char = CHAR(STRING_ELT(UPLO, 0));
  Integer    UPLO_length = 1;
  int        N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  double     *A_doubleMatrix = 0;

/*initialization for dimension variables*/
  Integer    LDA_integer = MAX(1, N_integer);
  Integer    LWORK_integer = MAX(1, 3*N_integer-1);

/*initialization for output variables*/
  double     *W_doubleMatrix = 0;
  int        INFO_integer;

/*allocation general variables*/
  Integer    A_dim1 = LDA_integer;
  Integer    A_dim2 = MAX(1, N_integer);
  if (!(A_doubleMatrix = (double *) malloc(A_dim1*A_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "A");
    }
  Integer    W_dim1 = MAX(1, N_integer);
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "W");
    }

/*initialization for workspace variables*/
  double     *WORK_doubleMatrix = 0;

/*allocation for workspace variables*/
  Integer    WORK_dim1 = MAX(1, LWORK_integer);
  if (!(WORK_doubleMatrix = (double *) malloc(WORK_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "WORK");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, A_dim1, A_dim2, A, A_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "A", nrows(A), A_dim1);
        }
      else
        {
          F08FAF_clean_up(A_doubleMatrix, W_doubleMatrix, WORK_doubleMatrix);
          error("Array extraction error for %s", "A");
        }
    }

  /* Call the NAG routine */
  IFAIL_integer = 1;
  f08faf_(JOBZ_char, UPLO_char, &N_integer, A_doubleMatrix, &LDA_integer,
          W_doubleMatrix, WORK_doubleMatrix, &LWORK_integer, &INFO_integer,
          JOBZ_length,
          UPLO_length);

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 3));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, A_dim1, A_dim2, A_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, W_dim1, 1, W_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &INFO_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 3));
  SET_STRING_ELT(names, 0, mkChar("A"));
  SET_STRING_ELT(names, 1, mkChar("W"));
  SET_STRING_ELT(names, 2, mkChar("INFO"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  F08FAF_clean_up(A_doubleMatrix, W_doubleMatrix, WORK_doubleMatrix);

  return(ans);
}

void F08FAF_clean_up(double *A, double *W, double *WORK)
{
  if (A) free(A);
  if (W) free(W);
  if (WORK) free(WORK);
  return;
}
